<?php
/**
 * @package     CSVI
 * @subpackage  K2
 *
 * @author      RolandD Cyber Produksi <contact@csvimproved.com>
 * @copyright   Copyright (C) 2006 - @year@ RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://csvimproved.com
 */

namespace k2\com_k2\model\import;

defined('_JEXEC') or die;

/**
 * Item import.
 *
 * @package     CSVI
 * @subpackage  K2
 * @since       6.0
 */
class Item extends \RantaiImportEngine
{
	/**
	 * Item table
	 *
	 * @var    \K2TableItem
	 * @since  6.0
	 */
	private $item;

	/**
	 * The addon helper
	 *
	 * @var    \Com_K2HelperCom_K2
	 * @since  6.0
	 */
	protected $helper;

	/**
	 * Custom fields array.
	 *
	 * @var    array
	 * @since  6.0
	 */
	private $customfields = array();

	/**
	 * Custom fields array.
	 *
	 * @var    \CsviHelperImportFields
	 * @since  6.0
	 */
	protected $fields;

	/**
	 * Extra field table
	 *
	 * @var    \K2TableExtrafield
	 * @since  6.5.0
	 */
	protected $extraField;

	/**
	 * Run this before we start.
	 *
	 * @return  void.
	 *
	 * @since   6.0
	 */
	public function onBeforeStart()
	{
		// Load the tables that will contain the data
		$this->loadCustomFields();
	}

	/**
	 * Start the item import process.
	 *
	 * @return  bool  True on success | false on failure.
	 *
	 * @since   6.0
	 */
	public function getStart()
	{
		// Process data
		foreach ($this->fields->getData() as $fields)
		{
			foreach ($fields as $name => $details)
			{
				$value = $details->value;

				switch ($name)
				{
					case 'created':
						$this->setState($name, $this->convertDate($value));
						break;
					case 'published':
						switch (strtoupper($value))
						{
							case 'Y':
								$value = 1;
								break;
							case 'N':
								$value = 0;
								break;
						}

						$this->setState($name, $value);
						break;
					default:
						$this->setState($name, $value);
						break;
				}
			}
		}

		// Check if we have a title and no alias, then we create the alias
		if (!$this->getState('alias', false) && $this->getState('title', false))
		{
			// Create the alias
			if (\JFactory::getConfig()->get('unicodeslugs') == 1)
			{
				$alias = \JFilterOutput::stringURLUnicodeSlug($this->getState('title'));
				$this->setState('alias', $alias);
			}
			else
			{
				// Detect the site language we will transliterate
				if (file_exists(JPATH_LIBRARIES . '/cms/component/helper.php'))
				{
					require_once JPATH_LIBRARIES . '/cms/component/helper.php';
				}

				$langParams  = \JComponentHelper::getParams('com_languages');
				$languageTag = $langParams->get('site');
				$language    = \JLanguage::getInstance($languageTag);
				$alias       = $language->transliterate($this->getState('title'));
				$alias       = \JFilterOutput::stringURLSafe($alias);

				if (trim(str_replace('-', '', $alias)) == '')
				{
					$alias = \JFactory::getDate()->format('Y-m-d-H-i-s');
				}

				$this->setState('alias', $alias);
			}

			// Need to add the field as it doesn't exist
			$data                = new \JObject;
			$data->field_name    = 'alias';
			$data->xml_node      = 'alias';
			$data->default_value = null;
			$data->enabled       = 1;
			$data->order         = 0;
			$data->supported     = 1;
			$data->value         = $alias;
			$this->fields->add($data);
		}

		// There must be an alias and catid or category_path
		if (!$this->getState('alias', false) && (!$this->getState('catid', false) && !$this->getState('category_path', false)))
		{
			$this->loaded = false;
		}
		else
		{
			// Find the content id
			$this->setState('id', $this->helper->getItemId());

			// Check if we have an existing item
			if ($this->getState('id', 0) > 0 && !$this->template->get('overwrite_existing_data'))
			{
				$this->log->add(\JText::sprintf('COM_CSVI_DATA_EXISTS_CONTENT', $this->getState('alias')));
				$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_CONTENT', $this->getState('alias')));
				$this->loaded = false;
			}
			else
			{
				// Load the current content data
				$this->item->load($this->getState('id', 0));
				$this->loaded = true;
			}
		}

		return true;
	}

	/**
	 * Process a record.
	 *
	 * @return  bool  Returns true if all is OK | Returns false if no product SKU or product ID can be found.
	 *
	 * @since   6.0
	 */
	public function getProcessRecord()
	{
		if ($this->loaded)
		{
			if (!$this->getState('id', false) && $this->template->get('ignore_non_exist'))
			{
				// Do nothing for new products when user chooses to ignore new products
				$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_IGNORE_NEW', $this->getState('alias', '')));
			}
			else
			{
				// Check if there is a category ID
				if (!$this->getState('catid', false) && $this->getState('category_path', false))
				{
					// Find the category ID
					$this->setState('catid', $this->helper->getCategoryIdByPath($this->getState('category_path')));
				}

				// Check if we have a title
				if (!$this->item->get('title', false))
				{
					$this->item->set('title', $this->getState('alias'));
				}

				// Prepare metadata field
				if (!$this->getState('metadata', false) && ($this->getState('metadata_robot', false) || $this->getState('metadata_author', false)))
				{
					// We create the field
					$values = array();
					$values[] = 'robots=' . $this->getState('metadata_robot', '');
					$values[] = 'author=' . $this->getState('metadata_author', '');

					$this->setState('metadata', implode("\n", $values));
				}

				// Set the modified date as we are modifying the product
				if (!$this->getState('modified', false))
				{
					$this->item->set('modified', $this->date->toSql());
					$this->item->set('modified_by', $this->userId);
				}

				// Check if the field is a custom field used as an available field
				$this->processCustomAvailableFields();

				// Set the plugin attributes
				$this->setPlugins();

				// Set some default values
				if (!$this->getState('id', false))
				{
					// Set a creation date and time if none specified
					if (!$this->getState('created'))
					{
						$this->item->created = $this->date->toSql();
					}

					if (!$this->getState('created_by'))
					{
						$this->item->created_by = $this->userId;
					}

					// Set the extra_fields value
					if (empty($this->item->extra_fields))
					{
						$this->setState('extra_fields', '[]');
					}

					// Set the param field
					$this->setState('params', $this->getState('params', '{"catItemTitle":"","catItemTitleLinked":"","catItemFeaturedNotice":"","catItemAuthor":"","catItemDateCreated":"","catItemRating":"","catItemImage":"","catItemIntroText":"","catItemExtraFields":"","catItemHits":"","catItemCategory":"","catItemTags":"","catItemAttachments":"","catItemAttachmentsCounter":"","catItemVideo":"","catItemVideoWidth":"","catItemVideoHeight":"","catItemAudioWidth":"","catItemAudioHeight":"","catItemVideoAutoPlay":"","catItemImageGallery":"","catItemDateModified":"","catItemReadMore":"","catItemCommentsAnchor":"","catItemK2Plugins":"","itemDateCreated":"","itemTitle":"","itemFeaturedNotice":"","itemAuthor":"","itemFontResizer":"","itemPrintButton":"","itemEmailButton":"","itemSocialButton":"","itemVideoAnchor":"","itemImageGalleryAnchor":"","itemCommentsAnchor":"","itemRating":"","itemImage":"","itemImgSize":"","itemImageMainCaption":"","itemImageMainCredits":"","itemIntroText":"","itemFullText":"","itemExtraFields":"","itemDateModified":"","itemHits":"","itemCategory":"","itemTags":"","itemAttachments":"","itemAttachmentsCounter":"","itemVideo":"","itemVideoWidth":"","itemVideoHeight":"","itemAudioWidth":"","itemAudioHeight":"","itemVideoAutoPlay":"","itemVideoCaption":"","itemVideoCredits":"","itemImageGallery":"","itemNavigation":"","itemComments":"","itemTwitterButton":"","itemFacebookButton":"","itemGooglePlusOneButton":"","itemAuthorBlock":"","itemAuthorImage":"","itemAuthorDescription":"","itemAuthorURL":"","itemAuthorEmail":"","itemAuthorLatest":"","itemAuthorLatestLimit":"","itemRelated":"","itemRelatedLimit":"","itemRelatedTitle":"","itemRelatedCategory":"","itemRelatedImageSize":"","itemRelatedIntrotext":"","itemRelatedFulltext":"","itemRelatedAuthor":"","itemRelatedMedia":"","itemRelatedImageGallery":"","itemK2Plugins":""}'));

					// Set the default metadata
					$this->setState('metadata', $this->getState('metadata', "robots=\r\nauthor="));

					// Set the default language
					$this->setState('language', $this->getState('language', '*'));
				}

				// Bind the initial data
				$this->item->bind($this->state);

				// Bind the data
				if ($this->item->store())
				{
					// Store values from content plugin
					$dispatcher = new \RantaiPluginDispatcher;
					$dispatcher->importPlugins('csviext', $this->db);

					// Fire the plugin to get attributes to import
					$dispatcher->trigger(
						'onAfterImport',
						array(
							'extension' => 'k2',
							'itemid' => $this->item->get('id'),
							'pluginfield' => $this->item->get('plugins', array()),
							'log'    => $this->log
						)
					);

					// Add the image if needed
					if ($this->getState('image', false))
					{
						$this->processMedia();
					}

					// Add the tags if needed
					if ($this->getState('tags', false))
					{
						$this->processTags();
					}
				}
				else
				{
					return false;
				}
			}

			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Load the necessary tables.
	 *
	 * @return  void.
	 *
	 * @since   6.0
	 */
	public function loadTables()
	{
		$this->item = $this->getTable('Item');
	}

	/**
	 * Clear the loaded tables.
	 *
	 * @return  void.
	 *
	 * @since   6.0
	 */
	public function clearTables()
	{
		$this->item->reset();
	}

	/**
	 * Process media files.
	 *
	 * @return  void.
	 *
	 * @since   4.0
	 */
	private function processMedia()
	{
		// Check if any image handling needs to be done
		if ($this->template->get('process_image', false))
		{
			// Image handling
			/** @var CsviHelperImage $imagehelper */
			$imagehelper = new \CsviHelperImage($this->template, $this->log, $this->csvihelper);

			if (\JFile::exists(JPATH_SITE . '/media/k2/items/src/' . trim($this->getState('image')))
				|| $imagehelper->isRemote($this->getState('image')))
			{
				// Load some helpers
				require_once JPATH_ADMINISTRATOR . '/components/com_k2/lib/class.upload.php';

				// Set the maximum sizes
				$max_width = $this->template->get('resize_max_width', 1024);
				$max_height = $this->template->get('resize_max_height', 768);

				// Get the full path and name of the image
				$filename = md5('Image' . $this->item->id);

				// Rename original image
				$image = JPATH_SITE . '/media/k2/items/src/' . $filename;

				// Check if we are dealing with a remote image
				if ($imagehelper->isRemote($this->getState('image')))
				{
					// Is a remote image, we must download it and store it locally
					$remote_image = str_replace(' ', '%20', $this->image);
					\JFile::write($image . '.jpg', file_get_contents($remote_image));
				}
				else
				{
					// Check if the image already exists
					$imageExists = \JFile::exists($image . '.jpg');

					if (!$imageExists || ($imageExists && $this->template->get('overwrite_image', false)))
					{
						\JFile::move(JPATH_SITE . '/media/k2/items/src/' . trim($this->getState('image')), $image . '.jpg');
					}
				}

				// Check if the thumbnail is not too big
				$thumb_sizes = getimagesize($image . '.jpg');

				if ($thumb_sizes[0] < $max_width || $thumb_sizes[1] < $max_height)
				{
					$imagehelper = new \Upload($image . '.jpg');

					// We need to create several thumbnails
					$sizes = array();
					$sizes['XS'] = $this->helperconfig->get('itemImageXS');
					$sizes['S'] = $this->helperconfig->get('itemImageS');
					$sizes['M'] = $this->helperconfig->get('itemImageM');
					$sizes['L'] = $this->helperconfig->get('itemImageL');
					$sizes['XL'] = $this->helperconfig->get('itemImageXL');
					$sizes['Generic'] = $this->helperconfig->get('itemImageGeneric');

					$savepath = JPATH_SITE . '/media/k2/items/cache';

					foreach ($sizes as $size => $width)
					{
						switch ($size)
						{
							default:
								$imagehelper->image_resize = true;
								$imagehelper->image_ratio_y = true;
								$imagehelper->image_convert = 'jpg';
								$imagehelper->jpeg_quality = $this->helperconfig->get('imagesQuality');
								$imagehelper->file_auto_rename = false;
								$imagehelper->file_overwrite = true;
								$imagehelper->file_new_name_body = $filename . '_' . $size;
								$imagehelper->image_x = $width;
								$imagehelper->Process($savepath);
								break;
						}
					}
				}
			}
		}
	}

	/**
	 * Get a list of custom fields that can be used as available field.
	 *
	 * @return  void.
	 *
	 * @since   4.4.1
	 */
	private function loadCustomFields()
	{
		$query = $this->db->getQuery(true)
			->select(
				array(
					$this->db->quoteName('x.id'),
					$this->db->quoteName('x.type'),
					'TRIM(
						CONCAT(' .
							$this->db->quoteName('g.name') . ', ' . $this->db->quote('_') . ', ' . $this->db->quoteName('x.name') .
						') ' .
					') AS ' . $this->db->quoteName('title')
				)
			)
			->from($this->db->quoteName('#__k2_extra_fields', 'x'))
			->leftJoin(
				$this->db->quoteName('#__k2_extra_fields_groups', 'g')
				. ' ON ' . $this->db->quoteName('g.id') . ' = ' . $this->db->quoteName('x.group')
			);
		$this->db->setQuery($query);
		$this->customfields = $this->db->loadObjectlist();

		$this->log->add('Load custom fields', true);
	}

	/**
	 * Process custom fields that are used as available field.
	 *
	 * @return  void.
	 *
	 * @since   4.4.1
	 *
	 * @throws  \RuntimeException
	 */
	private function processCustomAvailableFields()
	{
		// Create the queries
		if (0 !== count($this->customfields))
		{
			$processEmptyField = $this->template->get('emptyfield', false);

			foreach ($this->customfields as $field)
			{
				// Extract the group from the field
				$separatorPosition  = strpos($field->title, '_');
				$group              = substr($field->title, 0, $separatorPosition);
				$title              = substr($field->title, $separatorPosition + 1);
				$groupTitle         = $field->title;
				$importValue        = $this->getState($groupTitle, false);
				$this->log->add('Processing custom available field: ' . $title . ' in group ' . $group, false);

				if ($importValue || $processEmptyField)
				{
					// Load the existing value
					$add          = true;
					$value        = '';
					$extra_fields = json_decode($this->item->extra_fields);

					// Check if we need to do any formatting
					switch ($field->type)
					{
						case 'date':
							// Date format needs to be YYYY/MM/DD
							$value = $this->convertDate($importValue, 'date', 'user');
							break;
						case 'multipleSelect':
							// We need to find the correct values
							$query = $this->db->getQuery(true)
							->select($this->db->quoteName('value'))
							->from($this->db->quoteName('#__k2_extra_fields'))
							->where($this->db->quoteName('name') . ' = ' . $this->db->quote($title));
							$this->db->setQuery($query);
							$extra_values = json_decode($this->db->loadResult());

							// Multiple selects are | separated
							$values = explode('|', $importValue);
							$value = array();

							foreach ($extra_values as $extra_value)
							{
								if (in_array($extra_value->name, $values, true))
								{
									$value[] = $extra_value->value;
								}
							}

							break;
						case 'radio':
						case 'select':
							// We need to find the correct values
							$query = $this->db->getQuery(true)
								->select($this->db->quoteName('value'))
								->from($this->db->quoteName('#__k2_extra_fields'))
								->where($this->db->quoteName('name') . ' = ' . $this->db->quote($title));
							$this->db->setQuery($query);
							$extra_values = json_decode($this->db->loadResult());

							// Multiple selects are | separated
							foreach ($extra_values as $extra_value)
							{
								if ($extra_value->name === $importValue)
								{
									$value = (string) $extra_value->value;
								}
							}

							break;
						case 'link':
							$value = implode('|', $importValue);
							break;
						case 'textfield':
						case 'labels':
						default:
							$value = $importValue;
							break;
					}

					// Insert query if it is not empty
					if ('' !== $value || $processEmptyField)
					{
						if (is_array($extra_fields))
						{
							foreach ($extra_fields as $extra_field)
							{
								if ($field->id === $extra_field->id)
								{
									$extra_field->value = $value;
									$add = false;
								}
							}
						}

						if ($add)
						{
							$extra_field        = new \stdClass;
							$extra_field->id    = $field->id;
							$extra_field->value = $value;
							$extra_fields[]     = $extra_field;
						}

						// Store the new value
						$this->item->set('extra_fields', json_encode($extra_fields));
					}
					else
					{
						// If the value is not in extra fields table make a new entry
						$this->extraField = $this->getTable('Extrafield');
						$this->extraField->load($field->id);
						$existingValues          = json_decode($this->extraField->value, true);
						$value                   = array();
						$value['name']           = $importValue;
						$value['value']          = count($existingValues) + 1;
						$value['target']         = '';
						$value['alias']          = $title;
						$value['required']       = 0;
						$value['showNull']       = 1;
						$existingValues[]        = $value;
						$this->extraField->id    = $field->id;
						$this->extraField->value = json_encode($existingValues);
						$this->extraField->store();

						// Store the new value in items table
						if (is_array($extra_fields))
						{
							foreach ($extra_fields as $extra_field)
							{
								if ($field->id === $extra_field->id)
								{
									$extra_field->value = count($existingValues);
								}
							}
						}

						// Store the new value
						$this->item->set('extra_fields', json_encode($extra_fields));

					}
				}
			}
		}
		else
		{
			$this->log->add('No custom available fields found');
		}
	}

	/**
	 * Process item tags.
	 *
	 * @return  void.
	 *
	 * @since   6.5.0
	 *
	 * @throws  \RuntimeException
	 */
	private function processTags()
	{
		// Clean out all existing tags
		$query = $this->db->getQuery(true)
			->delete($this->db->quoteName('#__k2_tags_xref'))
			->where($this->db->quoteName('itemID') . ' = ' . (int) $this->item->get('id'));
		$this->db->setQuery($query)->execute();
		$this->log->add('Clean tags');

		$tags_import = $this->getState('tags', false);

		// Get the tags
		$tags = explode('|', $tags_import);

		// Process the tags
		foreach ($tags as $tag)
		{
			// Find the tag ID
			$query->clear()
				->select($this->db->quoteName('id'))
				->from($this->db->quoteName('#__k2_tags'))
				->where($this->db->quoteName('name') . ' = ' . $this->db->quote($tag));
			$this->db->setQuery($query);

			$tagId = $this->db->loadResult();

			$this->log->add('Find the tag ID');

			// Tag doesn't exist, let's create it
			if (!$tagId)
			{
				$query->clear()
					->insert($this->db->quoteName('#__k2_tags'))
					->columns($this->db->quoteName(array('name', 'published')))
					->values($this->db->quote($tag) . ', 1');
				$this->db->setQuery($query)->execute();

				$tagId = $this->db->insertid();
			}

			if ($tagId)
			{
				// Insert the cross-reference
				$query->clear()
					->insert($this->db->quoteName('#__k2_tags_xref'))
					->columns(array($this->db->quoteName('tagID'), $this->db->quoteName('itemID')))
					->values((int) $tagId . ',' . (int) $this->item->get('id'));
				$this->db->setQuery($query)->execute();

				$this->log->add('Insert the cross reference');
			}
		}
	}

	/**
	 * Set the plugins field.
	 *
	 * @return  void.
	 *
	 * @since   6.5.0
	 */
	public function setPlugins()
	{
		// Get value from content plugin
		$dispatcher = new \RantaiPluginDispatcher;
		$dispatcher->importPlugins('csviext', $this->db);

		// Fire the plugin to get plugin field to import
		$pluginFields = $dispatcher->trigger(
			'getAttributes',
			array(
				'extension' => 'k2',
				'operation' => 'item',
				'log'    => $this->log
			)
		);

		if (!empty($pluginFields[0]))
		{
			// Load the current plugins
			$plugins = json_decode($this->item->get('plugins'));

			if (!is_object($plugins))
			{
				$plugins = new \stdClass;
			}

			foreach ($pluginFields[0] as $plugin)
			{
				$plugins->$plugin = '';

				if ($this->getState($plugin, false))
				{
					$plugins->$plugin = $this->getState($plugin, '');
				}
			}

			// Store the new attributes
			$this->setState('plugins', json_encode($plugins));
			$this->log->add('Plugin field set for K2 swmap plugin', false);
		}
	}
}
